#!/bin/bash

PROJECT="kt-sysadmin"
FROM="kienan@koumbit.org"
REDMINE_USER="redmine"
REDMINE_HOST="redmine.koumbit.net"
POSTSCRIPT="Sysadmin Wiki to Redmine Migration $(date +%c)"
declare -A STATUSMAP=(
    ["en cours"]="In Progress"
    ["stagnant"]="In Progress"
    ["encours"]="In Progress"
    ["pas fait"]="New"
    ["à faire"]="New"
)
declare -A ASSIGNEEMAP=(
    ["alexandru"]="Alexandru Petrosan"
    ["dru"]="Alexandru Petrosan"
    ["pancake"]="Alexandru Petrosan"
    ["Kienan"]="Kienan Stewart"
    ["sebas"]="Sébastien Grenier"
    ["sébas"]="Sébastien Grenier"
    ["tous"]=""
    ["nizar"]="Nizar Koubaa"
    ["normand"]="Nizar Koubaa"
    ["guillaume"]="Guillaume Beaulieu"
    ["dan"]="Daniel KT"
    ["gab"]="Gabriel Filion"
    ["lutin"]="Gabriel Filion"
)

FILE="$1"
if [ ! -f "$1" ] || [ -z "$1" ] ; then
    echo "Source file $1 doesn't exist. Cannot continue."
    echo "Usage: $0 filename"
    exit 1
fi

function map_status {
    local OLD=$1
    local i
    local NEW=''
    for i in "${!STATUSMAP[@]}" ; do
        #echo "key  : $i"
        #echo "old  : $OLD"
        echo "$OLD" | grep -iq "$i"
        if [ "$?" -eq 0 ] ; then
            NEW="${STATUSMAP[$i]}"
            break
        fi
    done
    if [ -z "$NEW" ] ; then
        echo "New"
    else
        echo "$NEW"
    fi
}

function map_assignee {
    local OLD=$1
    local i
    local NEW=''
    for i in "${!ASSIGNEEMAP[@]}" ; do
        echo "$OLD" | grep -iq "$i"
        if [ "$?" -eq 0 ] ; then
            NEW="${ASSIGNEEMAP[$i]}"
            break
        fi
    done
    if [ -z "$NEW" ] ; then
        echo ""
    else
        echo "$NEW"
    fi
}

while IFS= read -r LINE ; do
    #echo '[DEBUG] Line is: ' "$LINE"
    ASSIGNEES=$(echo "$LINE" | grep -o ' Action:: \[.*\]' | grep -o '\[.*\]' | tr -d '[]')
    PRIMARY=$(echo "$ASSIGNEES" | cut -d ',' -f 1)
    OTHERS=$(echo "$ASSIGNEES" | cut -d ',' -s -f1 --complement)
    if [ "$OTHERS" == "$PRIMARY" ] ; then
        OTHERS=''
    fi
    #echo '[DEBUG] Assignees: ' "$ASSIGNEES"
    #echo '[DEBUG] Primary Assignee: ' "$PRIMARY"
    #echo '[DEBUG] Secondary Assignee(s): ' "$OTHERS"
    STATUS=$(echo "$LINE" | grep -o -e "'''.*'''\$" | tr -d "'")
    if [ -z "$STATUS" ] ; then
        STATUS='New'
    fi
    #echo '[DEBUG] Status: ' "$STATUS"
    SUBJECT=$(echo "$LINE" | sed "s/^ Action:: \[.*\] \(.*\)$/\1/")
    if [ "$STATUS" != "new" ] ; then
        # Remove the '''<status>''' from the subject
        SUBJECT=$(echo "$SUBJECT" | rev | cut -d "'" -f 1,2,3,4,5,6 --complement | rev)
    fi
    #echo '[DEBUG] Subject: ' "$SUBJECT"
    #echo "[DEBUG] Line2  :   Action:: [$PRIMARY, $OTHERS] $SUBJECT '''$STATUS'''"
    #map_status "$STATUS"
    MAPPEDSTATUS=$(map_status "$STATUS")
    MAPPEDASSIGNEE=$(map_assignee "$PRIMARY")
    #echo "[DEBUG] Mapped Status: ${MAPPEDSTATUS}"
    #echo "[DEBUG] Mapped Assignee: ${MAPPED_ASSIGNEE}"
    # Touch up the secondary assignees:
    if [ ! -z "$OTHERS" ] ; then
        OTHERS="Secondary assignees: $OTHERS"
    fi
    MAIL_BODY=$(cat <<EOF
Assigned to: $MAPPEDASSIGNEE
Status: $MAPPEDSTATUS
Tracker: Task
$OTHERS
Original action: <pre>$LINE</pre>

$POSTSCRIPT
EOF
             )
    #echo "[DEBUG] Mail body: ${MAIL_BODY}"
    TO="${REDMINE_USER}+${PROJECT}@${REDMINE_HOST}"
    #echo "[DEBUG] Command: swaks --from \"${FROM}\" --header-Subject \"${SUBJECT}\" --to \"$TO\" --body \"${MAIL_BODY}\""
    swaks --from "${FROM}" --header-Subject "${SUBJECT}" --to "$TO" --body "${MAIL_BODY}"
done < "$FILE"
